#ifndef TIMEDEPPOISSON1D_H
#define TIMEDEPPOISSON1D_H

#include "timedeppde1d.h"

typedef double (*Fct1D)(double);
typedef double (*TimedepFct1D)(double t, double x);

/** TimedepPoisson1D:
 *      Problem class that implements a 1D Laplace Operator with time-dependent 
 *      right-hand side and time-dependent Dirichlet boundary conditions
 *      
 *      Optionally with 
 *       - (non-homogeneous) Dirichlet boundary conditions
 *       - (non-homogeneous) initial values for non-stationary problems
 */
class TimedepPoisson1D : public TimedepPDE1D {
    
    TimedepFct1D f;             // Right hand side function, now f = f(t,x)
    TimedepFct1D d_bc;          // Dirichlet boundary condition function, d_bc = d_bc(t,x)
    Fct1D u0_fct;               // Initial value condition function
    

public:
    TimedepPoisson1D(TimedepFct1D _f);
    TimedepPoisson1D(TimedepFct1D _f, TimedepFct1D _d_bc);
    TimedepPoisson1D(TimedepFct1D _f, TimedepFct1D _d_bc, Fct1D _u0_fct);
    
    void
    assemble_on_element(double a, double b, double A[2][2], double b[2]);
    
    double
    dirichlet_bc(double x);

    double
    initial_condition(double x);

};

TimedepPoisson1D::TimedepPoisson1D(TimedepFct1D _f) : f(_f){}
TimedepPoisson1D::TimedepPoisson1D(TimedepFct1D _f, TimedepFct1D _d_bc) : f(_f), d_bc(_d_bc){}
TimedepPoisson1D::TimedepPoisson1D(TimedepFct1D _f, TimedepFct1D _d_bc, Fct1D _u0_fct) : f(_f), d_bc(_d_bc), u0_fct(_u0_fct){}


/********
 * TODO * 
 ********/
void
TimedepPoisson1D::assemble_on_element(double x_left, double x_right, double A[2][2], double b[2])
{
    /* Fill A and b with the correct values.
     * Basically the same as in Poisson1D, but 
     * f has to be called with 2 arguments now.
     */
}

double
TimedepPoisson1D::dirichlet_bc(double x){
    if(d_bc != NULL){
        return d_bc(current_time,x);
    }
    return 0;
};

double
TimedepPoisson1D::initial_condition(double x){
    if(u0_fct != NULL){
        return u0_fct(x);
    }
    else{
        return 0;
    }
};

#endif // TIMEDEPPOISSON_H