#ifndef PDE1D_H
#define PDE1D_H

/** PDE1D:
 *      Provides an interface for PDE problem classes.
 */
class PDE1D{

public:
    
    PDE1D(){}
    
    /** Assembles A^(k) and b^(k) on each interval [x_i, x_(i+1)]
     *  (Abstract function that has to be implemented in each derived class)
     */
    virtual void 
    assemble_on_element(double x_left, double x_right, double A[2][2], double b[2]) = 0;
    
    /** Returns the Dirichlet boundary condition at point x
     *  (can be overloaded, default are homogeneous boundaries)
     */
    virtual
    double
    dirichlet_bc(double x){
        return 0;
    };

    /** Returns the initial condition at point x.
     *  (necessary if we want to solve a instationary PDE)
     *  (can be overloaded, default is homogeneous initial condition)
     */
    virtual
    double
    initial_condition(double x){
        return 0;
    };

};


#endif // PDE1D_H