#include <iostream>
#include <vector>
#include <set>

#include "poisson1d.h"
#include "ellipticfem1d.h"

using namespace std;

// Problem right hand side
double
f(double x){
    return 1;
}

// Problem boundary condition
double
d_bc_fct(double x){
    return -0.5*x;
}

int main(){

    // Initialize 1D mesh with mesh points x_0,...,x_N
    // (=>  S_h = {psi_1,...,psi_(N-1)})
    int N = 101;
    vector<double> mesh(N);
    double h = 1./(double)(N-1);
    for(int i = 0; i < N; ++i){
        mesh[i] = i*h;
    }
    
    // Initialize Dirichlet boundary conditions
    set<int> d_bc;
    d_bc.insert(0);
    d_bc.insert(N-1);
    
    
    // Initialize Problem and FEM structure
    Poisson1D poisson(f, d_bc_fct);
    EllipticFEM1D fem1d(mesh, poisson, d_bc);
    
    // Solve and write solution to file
    fem1d.solve();
    fem1d.write_solution("u.txt");

    return 0;
}